import inspect
from functools import wraps
from typing import Any, Callable, Optional

import respx

from ._routes import (
    BetaRoutes,
    ChatRoutes,
    EmbeddingsRoutes,
    FileRoutes,
    ModelRoutes,
    ModerationsRoutes,
)
from .stores import StateStore


class OpenAIMock:
    beta: BetaRoutes
    chat: ChatRoutes
    embeddings: EmbeddingsRoutes
    files: FileRoutes
    models: ModelRoutes
    moderations: ModerationsRoutes

    def __init__(
        self,
        base_url: Optional[str] = None,
        state: Optional[StateStore] = None,
    ) -> None:
        self._router = respx.mock(
            assert_all_called=False,
            base_url=base_url or "https://api.openai.com/v1",
        )
        self._state = state or StateStore()
        self._init_routes()

    @property
    def router(self) -> respx.MockRouter:
        """[RESPX](https://lundberg.github.io/respx) router with patched OpenAI routes"""
        return self._router

    @property
    def state(self) -> StateStore:
        """State store for API resources"""
        return self._state

    @state.setter
    def state(self, value: StateStore) -> None:
        assert isinstance(value, StateStore), "Object is not an instance of StateStore"
        self._state = value
        self._init_routes()

    def _init_routes(self) -> None:
        """Called on construction and anytime that either `router` or `state` values are changed"""
        self.beta = BetaRoutes(self._router, self._state)
        self.chat = ChatRoutes(self._router)
        self.embeddings = EmbeddingsRoutes(self._router)
        self.files = FileRoutes(self._router, self._state)
        self.models = ModelRoutes(self._router, self._state)
        self.moderations = ModerationsRoutes(self._router)

        # NOTE: need to sort routes to avoid match conflicts
        self._router.routes._routes.sort(key=lambda r: len(repr(r._pattern)), reverse=True)  # type: ignore

    def _start_mock(self):
        def wrapper(fn: Callable[..., Any]):
            is_async = inspect.iscoroutinefunction(fn)
            argspec = inspect.getfullargspec(fn)
            needs_ref = "openai_mock" in argspec.args

            @wraps(fn)
            async def async_wrapper(*args: Any, **kwargs: Any):
                if needs_ref:
                    kwargs["openai_mock"] = self
                with self.router:
                    return await fn(*args, **kwargs)

            @wraps(fn)
            def sync_wrapper(*args: Any, **kwargs: Any):
                if needs_ref:
                    kwargs["openai_mock"] = self
                with self.router:
                    return fn(*args, **kwargs)

            return async_wrapper if is_async else sync_wrapper

        return wrapper



from typing import Annotated, Dict, List, Literal, TypedDict, Union
from typing_extensions import NotRequired

from openai._utils import PropertyInfo

__all__ = ["PartialMessage", "PartialMessageDeleted"]


class PartialFileSearchTool(TypedDict):
    type: Literal["file_search"]


class PartialCodeInterpreterTool(TypedDict):
    type: Literal["code_interpreter"]


class PartialAttachment(TypedDict):
    file_id: NotRequired[str]
    tools: NotRequired[List[Union[PartialCodeInterpreterTool, PartialFileSearchTool]]]


class PartialImageFile(TypedDict):
    file_id: str


class PartialImageFileContentBlock(TypedDict):
    type: Literal["image_file"]
    image_file: PartialImageFile


class PartialImageURL(TypedDict):
    url: str
    detail: NotRequired[Literal["auto", "low", "high"]]


class PartialImageURLContentBlock(TypedDict):
    type: Literal["image_url"]
    image_url: PartialImageURL


class PartialFileCitation(TypedDict):
    file_id: str
    quote: str


class PartialFileCitationAnnotation(TypedDict):
    end_index: int
    file_citation: PartialFileCitation
    start_index: int
    text: str
    type: Literal["file_citation"]


class PartialFilePath(TypedDict):
    file_id: str


class PartialFilePathAnnotation(TypedDict):
    end_index: int
    file_path: PartialFilePath
    start_index: int
    text: str
    type: Literal["file_path"]


class PartialText(TypedDict):
    annotations: List[
        Annotated[
            Union[PartialFileCitationAnnotation, PartialFilePathAnnotation],
            PropertyInfo(discriminator="type"),
        ]
    ]
    value: str


class PartialTextContentBlock(TypedDict):
    type: Literal["text"]
    text: PartialText


class PartialRefusalContentBlock(TypedDict):
    type: Literal["refusal"]
    refusal: str


PartialMessageContent = Annotated[
    Union[
        PartialImageFileContentBlock,
        PartialImageURLContentBlock,
        PartialTextContentBlock,
        PartialRefusalContentBlock,
    ],
    PropertyInfo(discriminator="type"),
]


class PartialMessage(TypedDict):
    id: NotRequired[str]
    assistant_id: NotRequired[str]
    attachments: NotRequired[List[PartialAttachment]]
    completed_at: NotRequired[int]
    content: NotRequired[
        List[
            Annotated[
                Union[PartialImageFileContentBlock, PartialTextContentBlock],
                PropertyInfo(discriminator="type"),
            ]
        ]
    ]
    created_at: NotRequired[int]
    incomplete_at: NotRequired[int]
    metadata: NotRequired[Dict[str, str]]
    object: NotRequired[Literal["thread.message"]]
    role: NotRequired[Literal["user", "assistant"]]
    run_id: NotRequired[str]
    status: NotRequired[Literal["in_progress", "incomplete", "completed"]]
    thread_id: NotRequired[str]


class PartialMessageDeleted(TypedDict):
    id: NotRequired[str]
    object: NotRequired[Literal["thread.message.deleted"]]
    deleted: NotRequired[bool]



from typing import Optional

import httpx
from openai.types.beta.vector_stores.vector_store_file import VectorStoreFile

from ._base import _generic_builder
from ..._routes.beta.vector_store_files import VectorStoreFileCreateRoute
from ..._types.partials.vector_store_files import PartialVectorStoreFile

__all__ = ["vector_store_file_from_create_request"]


def vector_store_file_from_create_request(
    request: httpx.Request,
    *,
    extra: Optional[PartialVectorStoreFile] = None,
) -> VectorStoreFile:
    return _generic_builder(VectorStoreFileCreateRoute, request, extra)



from typing import Generic, TypeVar
from typing_extensions import NotRequired, TypedDict

S = TypeVar("S")

__all__ = ["PartialResourceDeleted"]


class PartialResourceDeleted(TypedDict, Generic[S]):
    id: NotRequired[str]
    object: NotRequired[S]
    deleted: NotRequired[bool]



from typing import Any, Dict, Generic, List, Literal, Optional, TypeVar, Union

from openai.types import FileObject, Model

from openai.types.beta.assistant import Assistant
from openai.types.beta.thread import Thread
from openai.types.beta.threads.message import Message
from openai.types.beta.threads.run import Run
from openai.types.beta.threads.runs.run_step import RunStep

from openai.types.beta.vector_store import VectorStore
from openai.types.beta.vector_stores.vector_store_file import VectorStoreFile
from openai.types.beta.vector_stores.vector_store_file_batch import VectorStoreFileBatch

from .content_store import ContentStore
from .._constants import SYSTEM_MODELS
from .._utils.serde import model_parse

__all__ = ["StateStore"]

AnyModel = Union[
    FileObject,
    Assistant,
    Thread,
    Message,
    Run,
    RunStep,
    Model,
    VectorStore,
    VectorStoreFile,
    VectorStoreFileBatch,
]


M = TypeVar("M", bound=AnyModel)


class StateStore:
    def __init__(self) -> None:
        self.files = FileStore()
        self.models = ModelStore()
        self.beta = Beta()

    def _blind_put(self, resource: Union[AnyModel, Any]) -> None:
        if isinstance(resource, FileObject):
            self.files.put(resource)
        elif isinstance(resource, Assistant):
            self.beta.assistants.put(resource)
        elif isinstance(resource, Thread):
            self.beta.threads.put(resource)
        elif isinstance(resource, Message):
            self.beta.threads.messages.put(resource)
        elif isinstance(resource, Run):
            self.beta.threads.runs.put(resource)
        elif isinstance(resource, RunStep):
            self.beta.threads.runs.steps.put(resource)
        elif isinstance(resource, Model):
            self.models.put(resource)
        elif isinstance(resource, VectorStore):
            self.beta.vector_stores.put(resource)
        elif isinstance(resource, VectorStoreFile):
            self.beta.vector_stores.files.put(resource)
        elif isinstance(resource, VectorStoreFileBatch):
            self.beta.vector_stores.file_batches.put(resource)
        else:
            raise TypeError(f"Cannot put object of type {type(resource)} in store")


class Beta:
    def __init__(self) -> None:
        self.assistants = AssistantStore()
        self.threads = ThreadStore()
        self.vector_stores = VectorStoreStore()


class BaseStore(Generic[M]):
    def __init__(self) -> None:
        self._data: Dict[str, M] = {}

    def put(self, obj: M) -> None:
        self._data[obj.id] = obj

    def get(self, id: str) -> Optional[M]:
        return self._data.get(id)

    def delete(self, id: str) -> bool:
        if self._data.get(id):
            del self._data[id]
            return True
        else:
            return False


class FileStore(BaseStore[FileObject]):
    def __init__(self) -> None:
        super().__init__()
        self.content = ContentStore()

    def list(
        self,
        purpose: Optional[
            Literal[
                "fine-tune",
                "fine-tune-results",
                "assistants",
                "assistants_output",
            ]
        ] = None,
    ) -> List[FileObject]:
        files = list(self._data.values())
        if purpose:
            files = [file for file in files if file.purpose == purpose]
        return files


class ModelStore(BaseStore[Model]):
    def __init__(self) -> None:
        super().__init__()
        for model in SYSTEM_MODELS:
            self._data[str(model["id"])] = model_parse(Model, model)

    def list(self) -> List[Model]:
        return list(self._data.values())


class AssistantStore(BaseStore[Assistant]):
    def list(
        self,
        limit: Optional[str] = None,
        order: Optional[str] = None,
        after: Optional[str] = None,
        before: Optional[str] = None,
    ) -> List[Assistant]:
        limit = limit or "20"
        objs = list(self._data.values())
        objs = list(reversed(objs)) if (order or "desc") == "desc" else objs

        start_ix = 0
        if after:
            obj = self._data.get(after)
            if obj:
                start_ix = objs.index(obj) + 1

        end_ix = None
        if before:
            obj = self._data.get(before)
            if obj:
                end_ix = objs.index(obj)

        objs = objs[start_ix:end_ix]
        return objs[: int(limit)]


class ThreadStore(BaseStore[Thread]):
    def __init__(self) -> None:
        super().__init__()
        self.messages = MessageStore()
        self.runs = RunStore()


class MessageStore(BaseStore[Message]):
    def list(
        self,
        thread_id: str,
        limit: Optional[str] = None,
        order: Optional[str] = None,
        after: Optional[str] = None,
        before: Optional[str] = None,
        run_id: Optional[str] = None,
    ) -> List[Message]:
        limit = limit or "20"
        objs = [m for m in list(self._data.values()) if m.thread_id == thread_id]
        if run_id:
            objs = [obj for obj in objs if obj.run_id == run_id]
        objs = list(reversed(objs)) if (order or "desc") == "desc" else objs

        start_ix = 0
        if after:
            obj = self._data.get(after)
            if obj:
                start_ix = objs.index(obj) + 1

        end_ix = None
        if before:
            obj = self._data.get(before)
            if obj:
                end_ix = objs.index(obj)

        objs = objs[start_ix:end_ix]
        return objs[: int(limit)]


class RunStore(BaseStore[Run]):
    def __init__(self) -> None:
        super().__init__()
        self.steps = RunStepStore()

    def list(
        self,
        thread_id: str,
        limit: Optional[str] = None,
        order: Optional[str] = None,
        after: Optional[str] = None,
        before: Optional[str] = None,
    ) -> List[Run]:
        limit = limit or "20"
        objs = [m for m in list(self._data.values()) if m.thread_id == thread_id]
        objs = list(reversed(objs)) if (order or "desc") == "desc" else objs

        start_ix = 0
        if after:
            obj = self._data.get(after)
            if obj:
                start_ix = objs.index(obj) + 1

        end_ix = None
        if before:
            obj = self._data.get(before)
            if obj:
                end_ix = objs.index(obj)

        objs = objs[start_ix:end_ix]
        return objs[: int(limit)]


class RunStepStore(BaseStore[RunStep]):
    def list(
        self,
        thread_id: str,
        run_id: str,
        limit: Optional[str] = None,
        order: Optional[str] = None,
        after: Optional[str] = None,
        before: Optional[str] = None,
    ) -> List[RunStep]:
        limit = limit or "20"
        objs = [
            m
            for m in list(self._data.values())
            if m.thread_id == thread_id and m.run_id == run_id
        ]
        objs = list(reversed(objs)) if (order or "desc") == "desc" else objs

        start_ix = 0
        if after:
            obj = self._data.get(after)
            if obj:
                start_ix = objs.index(obj) + 1

        end_ix = None
        if before:
            obj = self._data.get(before)
            if obj:
                end_ix = objs.index(obj)

        objs = objs[start_ix:end_ix]
        return objs[: int(limit)]


class VectorStoreStore(BaseStore[VectorStore]):
    def __init__(self) -> None:
        super().__init__()
        self.files = VectorStoreFileStore()
        self.file_batches = VectorStoreFileBatchStore()

    def list(
        self,
        limit: Optional[str] = None,
        order: Optional[str] = None,
        after: Optional[str] = None,
        before: Optional[str] = None,
    ) -> List[VectorStore]:
        limit = limit or "20"
        objs = list(self._data.values())
        objs = list(reversed(objs)) if (order or "desc") == "desc" else objs

        start_ix = 0
        if after:
            obj = self._data.get(after)
            if obj:
                start_ix = objs.index(obj) + 1

        end_ix = None
        if before:
            obj = self._data.get(before)
            if obj:
                end_ix = objs.index(obj)

        objs = objs[start_ix:end_ix]
        return objs[: int(limit)]

    def list_files_for_batch(
        self,
        vector_store_id: str,
        batch_id: str,
        limit: Optional[str] = None,
        order: Optional[str] = None,
        after: Optional[str] = None,
        before: Optional[str] = None,
        filter: Optional[
            Literal["in_progress", "completed", "failed", "cancelled"]
        ] = None,
    ) -> List[VectorStoreFile]:
        limit = limit or "20"
        file_ids = self.file_batches.get_related_files(batch_id)
        objs = [
            m
            for m in self.files.list(vector_store_id, filter=filter)
            if m.vector_store_id == vector_store_id
        ]
        objs = [obj for obj in objs if obj.id in file_ids]
        objs = list(reversed(objs)) if (order or "desc") == "desc" else objs

        start_ix = 0
        if after:
            obj = self.files.get(after)
            if obj:
                start_ix = objs.index(obj) + 1

        end_ix = None
        if before:
            obj = self.files.get(before)
            if obj:
                end_ix = objs.index(obj)

        objs = objs[start_ix:end_ix]
        return objs[: int(limit)]


class VectorStoreFileStore(BaseStore[VectorStoreFile]):
    def list(
        self,
        vector_store_id: str,
        limit: Optional[str] = None,
        order: Optional[str] = None,
        after: Optional[str] = None,
        before: Optional[str] = None,
        filter: Optional[
            Literal["in_progress", "completed", "failed", "cancelled"]
        ] = None,
    ) -> List[VectorStoreFile]:
        limit = limit or "20"
        objs = [
            m for m in list(self._data.values()) if m.vector_store_id == vector_store_id
        ]
        if filter:
            objs = [obj for obj in objs if obj.status == filter]
        objs = list(reversed(objs)) if (order or "desc") == "desc" else objs

        start_ix = 0
        if after:
            obj = self._data.get(after)
            if obj:
                start_ix = objs.index(obj) + 1

        end_ix = None
        if before:
            obj = self._data.get(before)
            if obj:
                end_ix = objs.index(obj)

        objs = objs[start_ix:end_ix]
        return objs[: int(limit)]


class VectorStoreFileBatchStore(BaseStore[VectorStoreFileBatch]):
    def __init__(self) -> None:
        super().__init__()
        self._related_files: Dict[str, List[str]] = {}

    def get_related_files(self, batch_id: str) -> List[str]:
        return self._related_files.get(batch_id, [])

    def add_related_file(self, batch_id: str, file_id: str) -> None:
        if not self._related_files.get(batch_id):
            self._related_files[batch_id] = []
        self._related_files[batch_id].append(file_id)



from typing import Any
from typing_extensions import override

import httpx
import respx

from openai.types import Model
from openai.pagination import SyncCursorPage

from ._base import StatefulRoute

from ..stores import StateStore
from .._types.partials.models import PartialModel
from .._types.partials.sync_cursor_page import PartialSyncCursorPage

from .._utils.serde import model_dict


class ModelListRoute(
    StatefulRoute[SyncCursorPage[Model], PartialSyncCursorPage[PartialModel]]
):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.get(url__regex="/models"),
            status_code=200,
            state=state,
        )

    @override
    def _handler(self, request: httpx.Request, route: respx.Route) -> httpx.Response:
        self._route = route
        data = self._state.models.list()
        model = SyncCursorPage[Model](data=data)
        return httpx.Response(status_code=200, json=model_dict(model), request=request)

    @staticmethod
    def _build(
        partial: PartialSyncCursorPage[PartialModel],
        request: httpx.Request,
    ) -> SyncCursorPage[Model]:
        raise NotImplementedError


class ModelRetrieveRoute(StatefulRoute[Model, PartialModel]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.get(url__regex=r"/models/(?P<model_id>.+)"),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route
        model_id = kwargs["model_id"]
        found = self._state.models.get(model_id)
        if not found:
            return httpx.Response(404)

        return httpx.Response(status_code=200, json=model_dict(found), request=request)

    @staticmethod
    def _build(partial: PartialModel, request: httpx.Request) -> Model:
        raise NotImplementedError



from typing import Literal, TypedDict
from typing_extensions import NotRequired

__all__ = ["PartialVectorStoreFileBatch"]


class PartialFileCounts(TypedDict):
    cancelled: int
    completed: int
    failed: int
    in_progress: int
    total: int


class PartialVectorStoreFileBatch(TypedDict):
    id: NotRequired[str]
    created_at: NotRequired[int]
    file_counts: NotRequired[PartialFileCounts]
    object: NotRequired[Literal["vector_store.files_batch"]]
    status: NotRequired[Literal["in_progress", "completed", "cancelled", "failed"]]
    vector_store_id: NotRequired[str]



import respx

from ...stores import StateStore


from .assistants import (
    AssistantCreateRoute,
    AssistantListRoute,
    AssistantRetrieveRoute,
    AssistantUpdateRoute,
    AssistantDeleteRoute,
)
from .threads import (
    ThreadCreateRoute,
    ThreadRetrieveRoute,
    ThreadUpdateRoute,
    ThreadDeleteRoute,
)
from .messages import (
    MessageCreateRoute,
    MessageListRoute,
    MessageRetrieveRoute,
    MessageUpdateRoute,
    MessageDeleteRoute,
)
from .runs import (
    RunCreateRoute,
    ThreadCreateAndRun,
    RunListRoute,
    RunRetrieveRoute,
    RunUpdateRoute,
    RunSubmitToolOutputsRoute,
    RunCancelRoute,
)
from .run_steps import RunStepListRoute, RunStepRetrieveRoute
from .vector_stores import (
    VectorStoreCreateRoute,
    VectorStoreListRoute,
    VectorStoreRetrieveRoute,
    VectorStoreUpdateRoute,
    VectorStoreDeleteRoute,
)
from .vector_store_files import (
    VectorStoreFileCreateRoute,
    VectorStoreFileListRoute,
    VectorStoreFileRetrieveRoute,
    VectorStoreFileDeleteRoute,
)
from .vector_store_file_batches import (
    VectorStoreFileBatchCreateRoute,
    VectorStoreFileBatchRetrieveRoute,
    VectorStoreFileBatchCancelRoute,
    VectorStoreFileBatchListFilesRoute,
)

from .chat import ParsedChatCompletionsCreateRoute


__all__ = ["BetaRoutes"]


class BetaRoutes:
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        self.assistants = AssistantsRoutes(router, state)
        self.threads = ThreadRoutes(router, state)
        self.vector_stores = VectorStoreRoutes(router, state)

        self.chat = ChatRoutes(router)


class AssistantsRoutes:
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        self.create = AssistantCreateRoute(router, state)
        self.list = AssistantListRoute(router, state)
        self.retrieve = AssistantRetrieveRoute(router, state)
        self.update = AssistantUpdateRoute(router, state)
        self.delete = AssistantDeleteRoute(router, state)


class ThreadRoutes:
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        self.create = ThreadCreateRoute(router, state)
        self.retrieve = ThreadRetrieveRoute(router, state)
        self.update = ThreadUpdateRoute(router, state)
        self.delete = ThreadDeleteRoute(router, state)
        self.create_and_run = ThreadCreateAndRun(router, state)

        self.messages = MessageRoutes(router, state)
        self.runs = RunRoutes(router, state)


class MessageRoutes:
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        self.create = MessageCreateRoute(router, state)
        self.list = MessageListRoute(router, state)
        self.retrieve = MessageRetrieveRoute(router, state)
        self.update = MessageUpdateRoute(router, state)
        self.delete = MessageDeleteRoute(router, state)


class RunRoutes:
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        self.create = RunCreateRoute(router, state)
        self.list = RunListRoute(router, state)
        self.retrieve = RunRetrieveRoute(router, state)
        self.update = RunUpdateRoute(router, state)
        self.submit_tool_outputs = RunSubmitToolOutputsRoute(router, state)
        self.cancel = RunCancelRoute(router, state)

        self.steps = RunStepRoutes(router, state)


class RunStepRoutes:
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        self.list = RunStepListRoute(router, state)
        self.retrieve = RunStepRetrieveRoute(router, state)


class VectorStoreRoutes:
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        self.create = VectorStoreCreateRoute(router, state)
        self.list = VectorStoreListRoute(router, state)
        self.retrieve = VectorStoreRetrieveRoute(router, state)
        self.update = VectorStoreUpdateRoute(router, state)
        self.delete = VectorStoreDeleteRoute(router, state)

        self.files = VectorStoreFileRoutes(router, state)
        self.file_batches = VectorStoreFileBatchRoutes(router, state)


class VectorStoreFileRoutes:
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        self.create = VectorStoreFileCreateRoute(router, state)
        self.list = VectorStoreFileListRoute(router, state)
        self.retrieve = VectorStoreFileRetrieveRoute(router, state)
        self.delete = VectorStoreFileDeleteRoute(router, state)


class VectorStoreFileBatchRoutes:
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        self.create = VectorStoreFileBatchCreateRoute(router, state)
        self.retrieve = VectorStoreFileBatchRetrieveRoute(router, state)
        self.cancel = VectorStoreFileBatchCancelRoute(router, state)
        self.list_files = VectorStoreFileBatchListFilesRoute(router, state)


class ChatRoutes:
    def __init__(self, router: respx.MockRouter) -> None:
        self.completions = ParsedChatCompletionRoutes(router)


class ParsedChatCompletionRoutes:
    def __init__(self, router: respx.MockRouter) -> None:
        self.create = ParsedChatCompletionsCreateRoute(router)



import json
from typing import Any
from typing_extensions import override

import httpx
import respx

from openai.pagination import SyncCursorPage
from openai.types.beta.vector_stores.vector_store_file import VectorStoreFile
from openai.types.beta.vector_stores.vector_store_file_batch import VectorStoreFileBatch
from openai.types.beta.vector_stores.file_batch_create_params import (
    FileBatchCreateParams,
)


from .._base import StatefulRoute

from ...helpers.builders.vector_store_files import vector_store_file_from_create_request

from ...stores import StateStore
from ..._types.partials.sync_cursor_page import PartialSyncCursorPage
from ..._types.partials.vector_store_files import PartialVectorStoreFile
from ..._types.partials.vector_store_file_batches import PartialVectorStoreFileBatch

from ..._utils.faker import faker
from ..._utils.serde import json_loads, model_dict, model_parse
from ..._utils.time import utcnow_unix_timestamp_s

__all__ = [
    "VectorStoreFileBatchCreateRoute",
    "VectorStoreFileBatchRetrieveRoute",
    "VectorStoreFileBatchCancelRoute",
    "VectorStoreFileBatchListFilesRoute",
]


class VectorStoreFileBatchCreateRoute(
    StatefulRoute[VectorStoreFileBatch, PartialVectorStoreFileBatch]
):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.post(
                url__regex=r"/vector_stores/(?P<vector_store_id>[a-zA-Z0-9\_]+)/file_batches"
            ),
            status_code=201,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route

        vector_store_id = kwargs["vector_store_id"]
        found_vector_store = self._state.beta.vector_stores.get(vector_store_id)
        if not found_vector_store:
            return httpx.Response(404)

        model = self._build({"vector_store_id": vector_store_id}, request)
        content: FileBatchCreateParams = json_loads(request.content)
        for file_id in content["file_ids"]:
            found_file = self._state.files.get(file_id)
            if not found_file:
                return httpx.Response(404)
            encoded = json.dumps({"file_id": found_file.id})
            create_file_req = httpx.Request(method="", url="", content=encoded)
            vector_store_file = vector_store_file_from_create_request(
                create_file_req,
                extra={"vector_store_id": vector_store_id},
            )
            self._state.beta.vector_stores.files.put(vector_store_file)
            self._state.beta.vector_stores.file_batches.add_related_file(
                model.id,
                vector_store_file.id,
            )

        self._state.beta.vector_stores.file_batches.put(model)
        return httpx.Response(status_code=self._status_code, json=model_dict(model))

    @staticmethod
    def _build(
        partial: PartialVectorStoreFileBatch,
        request: httpx.Request,
    ) -> VectorStoreFileBatch:
        content = json_loads(request.content)
        defaults: PartialVectorStoreFileBatch = {
            "id": faker.beta.vector_store.file_batch.id(),
            "created_at": utcnow_unix_timestamp_s(),
            "file_counts": {
                "cancelled": 0,
                "completed": len(content["file_ids"]),
                "failed": 0,
                "in_progress": 0,
                "total": len(content["file_ids"]),
            },
            "object": "vector_store.files_batch",
            "status": "completed",
        }
        return model_parse(VectorStoreFileBatch, defaults | partial)


class VectorStoreFileBatchRetrieveRoute(
    StatefulRoute[VectorStoreFileBatch, PartialVectorStoreFileBatch]
):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.get(
                url__regex=r"/vector_stores/(?P<vector_store_id>[a-zA-Z0-9\_]+)/file_batches/(?P<batch_id>[a-zA-Z0-9\_]+)"
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route

        vector_store_id = kwargs["vector_store_id"]
        found_vector_store = self._state.beta.vector_stores.get(vector_store_id)
        if not found_vector_store:
            return httpx.Response(404)

        batch_id = kwargs["batch_id"]
        found = self._state.beta.vector_stores.file_batches.get(batch_id)
        if not found:
            return httpx.Response(404)

        return httpx.Response(status_code=self._status_code, json=model_dict(found))

    @staticmethod
    def _build(
        partial: PartialVectorStoreFileBatch,
        request: httpx.Request,
    ) -> VectorStoreFileBatch:
        raise NotImplementedError


class VectorStoreFileBatchCancelRoute(
    StatefulRoute[VectorStoreFileBatch, PartialVectorStoreFileBatch]
):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.post(
                url__regex=r"/vector_stores/(?P<vector_store_id>[a-zA-Z0-9\_]+)/file_batches/(?P<batch_id>[a-zA-Z0-9\_]+)/cancel"
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route

        vector_store_id = kwargs["vector_store_id"]
        found_vector_store = self._state.beta.vector_stores.get(vector_store_id)
        if not found_vector_store:
            return httpx.Response(404)

        batch_id = kwargs["batch_id"]
        found = self._state.beta.vector_stores.file_batches.get(batch_id)
        if not found:
            return httpx.Response(404)

        found.status = "cancelled"
        self._state.beta.vector_stores.file_batches.put(found)

        return httpx.Response(status_code=self._status_code, json=model_dict(found))

    @staticmethod
    def _build(
        partial: PartialVectorStoreFileBatch,
        request: httpx.Request,
    ) -> VectorStoreFileBatch:
        raise NotImplementedError


class VectorStoreFileBatchListFilesRoute(
    StatefulRoute[
        SyncCursorPage[VectorStoreFile], PartialSyncCursorPage[PartialVectorStoreFile]
    ]
):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.get(
                url__regex=r"/vector_stores/(?P<vector_store_id>[a-zA-Z0-9\_]+)/file_batches/(?P<batch_id>[a-zA-Z0-9\_]+)/files"
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route

        vector_store_id = kwargs["vector_store_id"]
        found_vector_store = self._state.beta.vector_stores.get(vector_store_id)
        if not found_vector_store:
            return httpx.Response(404)

        batch_id = kwargs["batch_id"]
        found = self._state.beta.vector_stores.file_batches.get(batch_id)
        if not found:
            return httpx.Response(404)

        limit = request.url.params.get("limit")
        order = request.url.params.get("order")
        after = request.url.params.get("after")
        before = request.url.params.get("before")
        filter = request.url.params.get("filter")

        data = self._state.beta.vector_stores.list_files_for_batch(
            vector_store_id,
            batch_id,
            limit,
            order,
            after,
            before,
            filter,
        )
        result_count = len(data)
        total_count = len(
            self._state.beta.vector_stores.list_files_for_batch(
                vector_store_id,
                batch_id,
            )
        )
        has_data = bool(result_count)
        has_more = total_count != result_count
        first_id = data[0].id if has_data else None
        last_id = data[-1].id if has_data else None
        model = SyncCursorPage[VectorStoreFile](data=data)
        return httpx.Response(
            status_code=200,
            json=model_dict(model)
            | {"first_id": first_id, "last_id": last_id, "has_more": has_more},
        )

    @staticmethod
    def _build(
        partial: PartialSyncCursorPage[PartialVectorStoreFile],
        request: httpx.Request,
    ) -> SyncCursorPage[VectorStoreFile]:
        raise NotImplementedError



from abc import ABC, abstractmethod
from functools import partial
import inspect
from typing import Any, Callable, Generic, Union
from typing_extensions import override

import httpx
import respx

from openai import BaseModel

from ..stores import StateStore
from .._types.generics import M, P
from .._utils.serde import model_dict

__all__ = ["StatelessRoute", "StatefulRoute"]


class Route(ABC, Generic[M, P]):
    def __init__(
        self,
        route: respx.Route,
        status_code: int,
    ) -> None:
        self._route = route
        self._status_code = status_code
        self._response: Union[httpx.Response, M, P, Callable[..., httpx.Response]] = (
            self._handler
        )
        self._route.side_effect = self._response

    @property
    def route(self) -> respx.Route:
        return self._route

    @property
    def response(self) -> Union[httpx.Response, M, P, Callable[..., httpx.Response]]:
        return self._response

    @response.setter
    def response(
        self,
        value: Union[httpx.Response, M, P, Callable[..., httpx.Response]],
    ) -> None:
        """
        Sets the value of route response. See docs for more details and examples.

        Args:
            value: Either an HTTPX response, an OpenAI model, a partial model, or a callable that returns an HTTPX response
        """
        self._response = value
        self._route.side_effect = self._side_effect

    @property
    def _side_effect(self) -> Callable[..., httpx.Response]:
        if callable(self._response):
            return self._response

        def _handler(request: httpx.Request, route: respx.Route, **kwargs: Any):
            if isinstance(self._response, BaseModel):
                return httpx.Response(
                    status_code=self._status_code,
                    json=model_dict(self._response),
                )

            elif isinstance(self._response, httpx.Response):
                return self._response

            else:
                assert not callable(self._response)
                return httpx.Response(
                    status_code=self._status_code,
                    json=model_dict(self._build(self._response, request), by_alias=True),
                )

        return _handler

    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
    ) -> httpx.Response:
        """Default response handler for route

        Args:
            request (httpx.Request): User request
            route (respx.Route): Associated route

        Returns:
            httpx.Response: Mocked response
        """
        self._route = route
        empty: Any = {}  # NOTE: avoids mypy complaint
        model = self._build(empty, request)
        return httpx.Response(
            status_code=self._status_code,
            json=model_dict(model),
        )

    @staticmethod
    @abstractmethod
    def _build(partial: P, request: httpx.Request) -> M:
        """Merge partial and content to create a full instance of model M

        Args:
            partial (P): Partial model
            content (bytes): Request content serializable to JSON

        Returns:
            M: Full model instance
        """
        raise NotImplementedError


class StatelessRoute(Route[M, P]):
    def __init__(self, *, route: respx.Route, status_code: int) -> None:
        super().__init__(route, status_code)


class StatefulRoute(Route[M, P]):
    def __init__(
        self,
        *,
        route: respx.Route,
        status_code: int,
        state: StateStore,
    ) -> None:
        super().__init__(route, status_code)
        self._state = state

    @property
    @override
    def _side_effect(self) -> Callable[..., httpx.Response]:
        if callable(self._response):
            argspec = inspect.getfullargspec(self._response)
            needs_store = (
                "state_store" in argspec.args or "state_store" in argspec.kwonlyargs
            )
            if needs_store:
                return partial(self._response, state_store=self._state)
            else:
                return self._response

        def _handler(request: httpx.Request, route: respx.Route, **kwargs: Any):
            if isinstance(self._response, BaseModel):
                self._state._blind_put(self._response)
                return httpx.Response(
                    status_code=self._status_code,
                    json=model_dict(self._response),
                )

            elif isinstance(self._response, httpx.Response):
                return self._response

            else:
                assert not callable(self._response)
                try:
                    model = self._build(self._response, request)
                    self._state._blind_put(model)
                    return httpx.Response(
                        status_code=self._status_code,
                        json=model_dict(model),
                    )
                except NotImplementedError:
                    import warnings

                    warnings.warn("Failed to build model")
                    warnings.warn("Falling back to default handler")
                    return self._handler(request, route)

        return _handler



import json
from typing import Any, List, Literal
from typing_extensions import override

import httpx
import respx

from openai.pagination import SyncCursorPage
from openai.types.beta.assistant import Assistant
from openai.types.beta.assistant_deleted import AssistantDeleted
from openai.types.beta.assistant_create_params import AssistantCreateParams
from openai.types.beta.assistant_update_params import AssistantUpdateParams

from .._base import StatefulRoute

from ...helpers.builders.vector_stores import vector_store_from_create_request
from ...helpers.builders.vector_store_files import vector_store_file_from_create_request
from ...helpers.mergers.assistants import merge_assistant_with_partial

from ...stores import StateStore

from ..._types.partials.deleted import PartialResourceDeleted
from ..._types.partials.sync_cursor_page import PartialSyncCursorPage
from ..._types.partials.assistants import PartialAssistant

from ..._utils.faker import faker
from ..._utils.serde import json_loads, model_dict, model_parse
from ..._utils.time import utcnow_unix_timestamp_s

__all__ = [
    "AssistantCreateRoute",
    "AssistantListRoute",
    "AssistantRetrieveRoute",
    "AssistantUpdateRoute",
    "AssistantDeleteRoute",
]


class AssistantCreateRoute(StatefulRoute[Assistant, PartialAssistant]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.post(url__regex="/assistants"),
            status_code=201,
            state=state,
        )

    @override
    def _handler(self, request: httpx.Request, route: respx.Route) -> httpx.Response:
        self._route = route
        model = self._build({}, request)
        content: AssistantCreateParams = json_loads(request.content)

        tool_resources = content.get("tool_resources")
        if tool_resources:
            file_search = tool_resources.get("file_search")
            if file_search:
                vector_stores = file_search.get("vector_stores")
                if vector_stores:
                    vector_store_ids: List[str] = []
                    for vector_store_create_params in vector_stores:
                        encoded = json.dumps(vector_store_create_params)
                        create_req = httpx.Request("", "", content=encoded)
                        vector_store = vector_store_from_create_request(create_req)
                        vector_store_ids.append(vector_store.id)
                        self._state.beta.vector_stores.put(vector_store)
                        for file_id in vector_store_create_params.get("file_ids", []):
                            found_file = self._state.files.get(file_id)
                            if not found_file:
                                return httpx.Response(404)
                            encoded = json.dumps({"file_id": found_file.id})
                            create_file_req = httpx.Request("", "", content=encoded)
                            vector_store_file = vector_store_file_from_create_request(
                                create_file_req,
                                extra={"vector_store_id": vector_store.id},
                            )
                            self._state.beta.vector_stores.files.put(vector_store_file)

                    model = merge_assistant_with_partial(
                        model,
                        {
                            "tool_resources": {
                                "file_search": {
                                    "vector_store_ids": vector_store_ids,
                                }
                            }
                        },
                    )

        self._state.beta.assistants.put(model)
        return httpx.Response(
            status_code=self._status_code,
            json=model_dict(model),
        )

    @staticmethod
    def _build(partial: PartialAssistant, request: httpx.Request) -> Assistant:
        content = json_loads(request.content)
        defaults: PartialAssistant = {
            "id": faker.beta.assistant.id(),
            "created_at": utcnow_unix_timestamp_s(),
            "tools": [],
            "object": "assistant",
        }
        return model_parse(Assistant, defaults | partial | content)


class AssistantListRoute(
    StatefulRoute[SyncCursorPage[Assistant], PartialSyncCursorPage[PartialAssistant]]
):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.get(url__regex="/assistants"),
            status_code=200,
            state=state,
        )

    @override
    def _handler(self, request: httpx.Request, route: respx.Route) -> httpx.Response:
        self._route = route

        limit = request.url.params.get("limit")
        order = request.url.params.get("order")
        after = request.url.params.get("after")
        before = request.url.params.get("before")

        data = self._state.beta.assistants.list(limit, order, after, before)
        result_count = len(data)
        total_count = len(self._state.beta.assistants.list())
        has_data = bool(result_count)
        has_more = total_count != result_count
        first_id = data[0].id if has_data else None
        last_id = data[-1].id if has_data else None
        model = SyncCursorPage[Assistant](data=data)
        return httpx.Response(
            status_code=200,
            json=model_dict(model)
            | {"first_id": first_id, "last_id": last_id, "has_more": has_more},
        )

    @staticmethod
    def _build(
        partial: PartialSyncCursorPage[PartialAssistant],
        request: httpx.Request,
    ) -> SyncCursorPage[Assistant]:
        raise NotImplementedError


class AssistantRetrieveRoute(StatefulRoute[Assistant, PartialAssistant]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.get(
                url__regex=r"/assistants/(?P<assistant_id>[a-zA-Z0-9\_]+)"
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route
        assistant_id = kwargs["assistant_id"]
        found = self._state.beta.assistants.get(assistant_id)
        if not found:
            return httpx.Response(404)

        return httpx.Response(status_code=200, json=model_dict(found))

    @staticmethod
    def _build(partial: PartialAssistant, request: httpx.Request) -> Assistant:
        raise NotImplementedError


class AssistantUpdateRoute(StatefulRoute[Assistant, PartialAssistant]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.post(
                url__regex=r"/assistants/(?P<assistant_id>[a-zA-Z0-9\_]+)"
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route
        assistant_id = kwargs["assistant_id"]
        found = self._state.beta.assistants.get(assistant_id)
        if not found:
            return httpx.Response(404)

        content: AssistantUpdateParams = json_loads(request.content)
        deserialized = model_dict(found)
        updated = model_parse(Assistant, deserialized | content)
        self._state.beta.assistants.put(updated)

        return httpx.Response(status_code=200, json=model_dict(updated))

    @staticmethod
    def _build(partial: PartialAssistant, request: httpx.Request) -> Assistant:
        raise NotImplementedError


class AssistantDeleteRoute(
    StatefulRoute[
        AssistantDeleted, PartialResourceDeleted[Literal["assistant.deleted"]]
    ]
):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.delete(
                url__regex=r"/assistants/(?P<assistant_id>[a-zA-Z0-9\_]+)"
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route
        assistant_id = kwargs["assistant_id"]
        deleted = self._state.beta.assistants.delete(assistant_id)
        return httpx.Response(
            status_code=200,
            json=model_dict(
                AssistantDeleted(
                    id=assistant_id, deleted=deleted, object="assistant.deleted"
                )
            ),
        )

    @staticmethod
    def _build(
        partial: PartialResourceDeleted[Literal["assistant.deleted"]],
        request: httpx.Request,
    ) -> AssistantDeleted:
        raise NotImplementedError



from typing import Optional

import httpx

from openai.types.create_embedding_response import CreateEmbeddingResponse

from ._base import _generic_builder
from ..._routes.embeddings import EmbeddingsCreateRoute
from ..._types.partials.embeddings import PartialCreateEmbeddingResponse

__all__ = ["embedding_create_response_from_create_request"]


def embedding_create_response_from_create_request(
    request: httpx.Request,
    *,
    extra: Optional[PartialCreateEmbeddingResponse] = None,
) -> CreateEmbeddingResponse:
    return _generic_builder(EmbeddingsCreateRoute, request, extra)



import json
from typing import Any, List, Literal
from typing_extensions import override

import httpx
import respx

from openai.types.beta.thread import Thread
from openai.types.beta.thread_create_params import ThreadCreateParams
from openai.types.beta.thread_update_params import ThreadUpdateParams
from openai.types.beta.thread_deleted import ThreadDeleted

from .._base import StatefulRoute

from ...helpers.builders.messages import message_from_create_request
from ...helpers.builders.vector_stores import vector_store_from_create_request
from ...helpers.builders.vector_store_files import vector_store_file_from_create_request
from ...helpers.mergers.threads import merge_thread_with_partial

from ...stores import StateStore
from ..._types.partials.deleted import PartialResourceDeleted
from ..._types.partials.threads import PartialThread

from ..._utils.faker import faker
from ..._utils.serde import json_loads, model_dict, model_parse
from ..._utils.time import utcnow_unix_timestamp_s


__all__ = [
    "ThreadCreateRoute",
    "ThreadRetrieveRoute",
    "ThreadUpdateRoute",
    "ThreadDeleteRoute",
]


class ThreadCreateRoute(StatefulRoute[Thread, PartialThread]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.post(url__regex="/threads"),
            status_code=201,
            state=state,
        )

    @override
    def _handler(self, request: httpx.Request, route: respx.Route) -> httpx.Response:
        self._route = route

        content: ThreadCreateParams = json_loads(request.content)
        model = self._build({}, request)
        self._state.beta.threads.put(model)

        for message_create_params in content.get("messages", []):
            encoded = json.dumps(message_create_params).encode("utf-8")
            create_message_req = httpx.Request(method="", url="", content=encoded)
            message = message_from_create_request(model.id, create_message_req)
            self._state.beta.threads.messages.put(message)

        tool_resources = content.get("tool_resources")
        if tool_resources:
            file_search = tool_resources.get("file_search")
            if file_search:
                vector_stores = file_search.get("vector_stores")
                if vector_stores:
                    vector_store_ids: List[str] = []
                    for vector_store_create_params in vector_stores:
                        encoded = json.dumps(vector_store_create_params)  # type: ignore
                        create_req = httpx.Request("", "", content=encoded)
                        vector_store = vector_store_from_create_request(create_req)
                        vector_store_ids.append(vector_store.id)
                        self._state.beta.vector_stores.put(vector_store)
                        for file_id in vector_store_create_params.get("file_ids", []):
                            found_file = self._state.files.get(file_id)
                            if not found_file:
                                return httpx.Response(404)
                            encoded = json.dumps({"file_id": found_file.id})  # type: ignore
                            create_file_req = httpx.Request("", "", content=encoded)
                            vector_store_file = vector_store_file_from_create_request(
                                create_file_req,
                                extra={"vector_store_id": vector_store.id},
                            )
                            self._state.beta.vector_stores.files.put(vector_store_file)

                    model = merge_thread_with_partial(
                        model,
                        {
                            "tool_resources": {
                                "file_search": {
                                    "vector_store_ids": vector_store_ids,
                                }
                            }
                        },
                    )

        return httpx.Response(
            status_code=self._status_code,
            json=model_dict(model),
        )

    @staticmethod
    def _build(partial: PartialThread, request: httpx.Request) -> Thread:
        content = json.loads(request.content)
        if content.get("messages"):
            del content["messages"]
        if content.get("tool_resources"):
            del content["tool_resources"]
        defaults: PartialThread = {
            "id": faker.beta.thread.id(),
            "created_at": utcnow_unix_timestamp_s(),
            "object": "thread",
        }
        return model_parse(Thread, defaults | partial | content)


class ThreadRetrieveRoute(StatefulRoute[Thread, PartialThread]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.get(url__regex=r"/threads/(?P<thread_id>[a-zA-Z0-9\_]+)"),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route
        thread_id = kwargs["thread_id"]
        found = self._state.beta.threads.get(thread_id)
        if not found:
            return httpx.Response(404)

        return httpx.Response(status_code=200, json=model_dict(found))

    @staticmethod
    def _build(partial: PartialThread, request: httpx.Request) -> Thread:
        raise NotImplementedError


class ThreadUpdateRoute(StatefulRoute[Thread, PartialThread]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.post(
                url__regex=r"/threads/(?P<thread_id>(?!.*runs)[a-zA-Z0-9_]+)"  # NOTE: avoids match on /threads/runs
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route
        thread_id = kwargs["thread_id"]
        found = self._state.beta.threads.get(thread_id)
        if not found:
            return httpx.Response(404)

        content: ThreadUpdateParams = json.loads(request.content)
        deserialized = model_dict(found)
        updated = model_parse(Thread, deserialized | content)
        self._state.beta.threads.put(updated)

        return httpx.Response(status_code=200, json=model_dict(updated))

    @staticmethod
    def _build(partial: PartialThread, request: httpx.Request) -> Thread:
        raise NotImplementedError


class ThreadDeleteRoute(
    StatefulRoute[ThreadDeleted, PartialResourceDeleted[Literal["thread.deleted"]]]
):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.delete(url__regex=r"/threads/(?P<thread_id>[a-zA-Z0-9\_]+)"),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route
        thread_id = kwargs["thread_id"]
        deleted = self._state.beta.threads.delete(thread_id)
        return httpx.Response(
            status_code=200,
            json=model_dict(
                ThreadDeleted(id=thread_id, deleted=deleted, object="thread.deleted")
            ),
        )

    @staticmethod
    def _build(
        partial: PartialResourceDeleted[Literal["thread.deleted"]],
        request: httpx.Request,
    ) -> ThreadDeleted:
        raise NotImplementedError



from typing import Generic, List, Literal, TypeVar
from typing_extensions import NotRequired, TypedDict

M = TypeVar("M")

__all__ = ["PartialSyncCursorPage"]


class PartialSyncCursorPage(TypedDict, Generic[M]):
    object: NotRequired[Literal["list"]]
    data: NotRequired[List[M]]
    first_id: NotRequired[str]
    last_id: NotRequired[str]
    has_more: NotRequired[bool]



from typing import List, TypedDict
from typing_extensions import NotRequired

__all__ = [
    "PartialModerationCreateResponse",
    "PartialCategories",
    "PartialCategoryScores",
]

PartialCategories = TypedDict(
    "PartialCategories",
    {
        "harassment": NotRequired[bool],
        "harassment/threatening": NotRequired[bool],
        "hate": NotRequired[bool],
        "hate/threatening": NotRequired[bool],
        "self-harm": NotRequired[bool],
        "self-harm/instructions": NotRequired[bool],
        "self-harm/intent": NotRequired[bool],
        "sexual": NotRequired[bool],
        "sexual/minors": NotRequired[bool],
        "violence": NotRequired[bool],
        "violence/graphic": NotRequired[bool],
    },
)

PartialCategoryScores = TypedDict(
    "PartialCategoryScores",
    {
        "harassment": NotRequired[float],
        "harassment/threatening": NotRequired[float],
        "hate": NotRequired[float],
        "hate/threatening": NotRequired[float],
        "self-harm": NotRequired[float],
        "self-harm/instructions": NotRequired[float],
        "self-harm/intent": NotRequired[float],
        "sexual": NotRequired[float],
        "sexual/minors": NotRequired[float],
        "violence": NotRequired[float],
        "violence/graphic": NotRequired[float],
    },
)


class PartialModeration(TypedDict):
    categories: NotRequired[PartialCategories]
    category_scores: NotRequired[PartialCategoryScores]
    flagged: NotRequired[bool]


class PartialModerationCreateResponse(TypedDict):
    id: NotRequired[str]
    model: NotRequired[str]
    results: NotRequired[List[PartialModeration]]



from typing import Optional

import httpx

from openai.types.beta.thread import Thread

from ._base import _generic_builder
from ..._routes.beta.threads import ThreadCreateRoute
from ..._types.partials.threads import PartialThread

__all__ = ["thread_from_create_request"]


def thread_from_create_request(
    request: httpx.Request,
    *,
    extra: Optional[PartialThread] = None,
) -> Thread:
    return _generic_builder(ThreadCreateRoute, request, extra)



from ._api import mock
from ._mock import OpenAIMock

from . import ext
from . import helpers
from . import stores
from . import streaming

__all__ = ["mock", "OpenAIMock", "ext", "helpers", "stores", "streaming"]



from typing import Optional

import httpx

from openai.types.chat.chat_completion import ChatCompletion

from ._base import _generic_builder
from ..._routes.chat import ChatCompletionsCreateRoute
from ..._types.partials.chat import PartialChatCompletion

__all__ = ["chat_completion_from_create_request"]


def chat_completion_from_create_request(
    request: httpx.Request,
    *,
    extra: Optional[PartialChatCompletion] = None,
) -> ChatCompletion:
    return _generic_builder(ChatCompletionsCreateRoute, request, extra)



from typing import Dict, Literal, TypedDict
from typing_extensions import NotRequired

__all__ = ["PartialVectorStore", "PartialVectorStoreDeleted"]


class PartialFileCount(TypedDict):
    cancelled: int
    completed: int
    failed: int
    in_progress: int
    total: int


class PartialExpiresAfter(TypedDict):
    anchor: Literal["last_active_at"]
    days: int


class PartialVectorStore(TypedDict):
    id: NotRequired[str]
    created_at: NotRequired[int]
    file_counts: NotRequired[PartialFileCount]
    last_active_at: NotRequired[int]
    metadata: NotRequired[Dict[str, str]]
    name: NotRequired[str]
    object: NotRequired[Literal["vector_store"]]
    status: NotRequired[Literal["expired", "in_progress", "completed"]]
    usage_bytes: NotRequired[int]
    expires_after: NotRequired[PartialExpiresAfter]
    expires_at: NotRequired[int]


class PartialVectorStoreDeleted(TypedDict):
    id: NotRequired[str]
    object: NotRequired[Literal["vector_store.deleted"]]
    deleted: NotRequired[bool]



from typing import Literal, TypedDict

from typing_extensions import NotRequired

__all__ = ["PartialVectorStoreFile"]


class PartialLastError(TypedDict):
    code: Literal[
        "internal_error",
        "file_not_found",
        "parsing_error",
        "unhandled_mime_type",
    ]
    message: str


class PartialVectorStoreFile(TypedDict):
    id: NotRequired[str]
    created_at: NotRequired[int]
    last_error: NotRequired[PartialLastError]
    object: NotRequired[Literal["vector_store.file"]]
    status: NotRequired[Literal["in_progress", "completed", "cancelled", "failed"]]
    usage_bytes: NotRequired[int]
    vector_store_id: NotRequired[str]



from typing import Any, Awaitable, Callable, Optional, Union

from ._mock import OpenAIMock
from .stores import StateStore

WrappedFn = Callable[..., Union[Callable[..., Any], Awaitable[Callable[..., Any]]]]


def mock(
    *,
    base_url: Optional[str] = None,
    state: Optional[StateStore] = None,
) -> WrappedFn:
    """
    Args:
        base_url (Optional[str], optional): Override base URL. Defaults to None.
        state (Optional[StateStore], optional): Override default empty state. Defaults to None.
    """
    openai_mock = OpenAIMock(base_url, state)
    return openai_mock._start_mock()



from typing import Optional

import httpx

from openai.types.moderation_create_response import ModerationCreateResponse

from ._base import _generic_builder
from ..._routes.moderation import ModerationCreateRoute
from ..._types.partials.moderation import PartialModerationCreateResponse

__all__ = ["moderation_create_response_from_create_request"]


def moderation_create_response_from_create_request(
    request: httpx.Request,
    *,
    extra: Optional[PartialModerationCreateResponse] = None,
) -> ModerationCreateResponse:
    return _generic_builder(ModerationCreateRoute, request, extra)



from typing import Any, Mapping, TypeVar

from openai import BaseModel

__all__ = ["M", "P"]

M = TypeVar("M", bound=BaseModel)
"""OpenAI model generic"""

P = TypeVar("P", bound=Mapping[str, Any])
"""Partial OpenAI model generic"""
